/********************************************************************************
** Form generated from reading UI file 'settings.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGS_H
#define UI_SETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "config/configdialog_p.h"
#include "kcolorbutton.h"
#include "kcombobox.h"

QT_BEGIN_NAMESPACE

class Ui_Settings
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_3;
    Palapeli::TriggerComboBox *kcfg_ViewBackground;
    KColorButton *kcfg_ViewBackgroundColor;
    QLabel *label_7;
    KColorButton *kcfg_ViewHighlightColor;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_5;
    QComboBox *kcfg_SolutionArea;
    QLabel *label_6;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_8;
    QSpacerItem *horizontalSpacer_2;
    QSlider *kcfg_PieceSpacing;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QCheckBox *kcfg_PieceBevelsEnabled;
    QCheckBox *kcfg_PieceShadowsEnabled;
    QLabel *label_4;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout_3;
    QLabel *label_1;
    QSlider *kcfg_SnappingPrecision;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSpacerItem *horizontalSpacer;
    QLabel *label_2;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *Settings)
    {
        if (Settings->objectName().isEmpty())
            Settings->setObjectName("Settings");
        Settings->resize(551, 519);
        verticalLayout_2 = new QVBoxLayout(Settings);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox_2 = new QGroupBox(Settings);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout = new QGridLayout(groupBox_2);
        gridLayout->setObjectName("gridLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_3 = new QLabel(groupBox_2);
        label_3->setObjectName("label_3");

        horizontalLayout_2->addWidget(label_3);

        kcfg_ViewBackground = new Palapeli::TriggerComboBox(groupBox_2);
        kcfg_ViewBackground->setObjectName("kcfg_ViewBackground");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(kcfg_ViewBackground->sizePolicy().hasHeightForWidth());
        kcfg_ViewBackground->setSizePolicy(sizePolicy);

        horizontalLayout_2->addWidget(kcfg_ViewBackground);


        gridLayout->addLayout(horizontalLayout_2, 0, 0, 1, 1);

        kcfg_ViewBackgroundColor = new KColorButton(groupBox_2);
        kcfg_ViewBackgroundColor->setObjectName("kcfg_ViewBackgroundColor");
        kcfg_ViewBackgroundColor->setProperty("defaultColor", QVariant(QColor(255, 255, 255)));

        gridLayout->addWidget(kcfg_ViewBackgroundColor, 0, 1, 1, 1);

        label_7 = new QLabel(groupBox_2);
        label_7->setObjectName("label_7");

        gridLayout->addWidget(label_7, 1, 0, 1, 1);

        kcfg_ViewHighlightColor = new KColorButton(groupBox_2);
        kcfg_ViewHighlightColor->setObjectName("kcfg_ViewHighlightColor");
        kcfg_ViewHighlightColor->setProperty("defaultColor", QVariant(QColor(255, 255, 255)));

        gridLayout->addWidget(kcfg_ViewHighlightColor, 1, 1, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_5 = new QLabel(groupBox_2);
        label_5->setObjectName("label_5");

        horizontalLayout_3->addWidget(label_5);

        kcfg_SolutionArea = new QComboBox(groupBox_2);
        kcfg_SolutionArea->setObjectName("kcfg_SolutionArea");
        sizePolicy.setHeightForWidth(kcfg_SolutionArea->sizePolicy().hasHeightForWidth());
        kcfg_SolutionArea->setSizePolicy(sizePolicy);

        horizontalLayout_3->addWidget(kcfg_SolutionArea);


        gridLayout->addLayout(horizontalLayout_3, 2, 0, 1, 1);

        label_6 = new QLabel(groupBox_2);
        label_6->setObjectName("label_6");
        label_6->setWordWrap(true);

        gridLayout->addWidget(label_6, 3, 0, 1, 2);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_8 = new QLabel(groupBox_2);
        label_8->setObjectName("label_8");

        horizontalLayout_4->addWidget(label_8);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_2);

        kcfg_PieceSpacing = new QSlider(groupBox_2);
        kcfg_PieceSpacing->setObjectName("kcfg_PieceSpacing");
        kcfg_PieceSpacing->setMinimum(0);
        kcfg_PieceSpacing->setMaximum(10);
        kcfg_PieceSpacing->setPageStep(1);
        kcfg_PieceSpacing->setValue(6);
        kcfg_PieceSpacing->setOrientation(Qt::Horizontal);
        kcfg_PieceSpacing->setTickPosition(QSlider::TicksBelow);
        kcfg_PieceSpacing->setTickInterval(2);

        horizontalLayout_4->addWidget(kcfg_PieceSpacing);


        gridLayout->addLayout(horizontalLayout_4, 4, 0, 1, 2);


        verticalLayout_2->addWidget(groupBox_2);

        groupBox = new QGroupBox(Settings);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        kcfg_PieceBevelsEnabled = new QCheckBox(groupBox);
        kcfg_PieceBevelsEnabled->setObjectName("kcfg_PieceBevelsEnabled");
        kcfg_PieceBevelsEnabled->setChecked(true);

        verticalLayout->addWidget(kcfg_PieceBevelsEnabled);

        kcfg_PieceShadowsEnabled = new QCheckBox(groupBox);
        kcfg_PieceShadowsEnabled->setObjectName("kcfg_PieceShadowsEnabled");
        kcfg_PieceShadowsEnabled->setChecked(true);

        verticalLayout->addWidget(kcfg_PieceShadowsEnabled);

        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");
        label_4->setWordWrap(true);

        verticalLayout->addWidget(label_4);


        verticalLayout_2->addWidget(groupBox);

        groupBox_3 = new QGroupBox(Settings);
        groupBox_3->setObjectName("groupBox_3");
        gridLayout_3 = new QGridLayout(groupBox_3);
        gridLayout_3->setObjectName("gridLayout_3");
        label_1 = new QLabel(groupBox_3);
        label_1->setObjectName("label_1");

        gridLayout_3->addWidget(label_1, 0, 0, 1, 1);

        kcfg_SnappingPrecision = new QSlider(groupBox_3);
        kcfg_SnappingPrecision->setObjectName("kcfg_SnappingPrecision");
        kcfg_SnappingPrecision->setMinimum(10);
        kcfg_SnappingPrecision->setMaximum(30);
        kcfg_SnappingPrecision->setPageStep(5);
        kcfg_SnappingPrecision->setValue(20);
        kcfg_SnappingPrecision->setOrientation(Qt::Horizontal);
        kcfg_SnappingPrecision->setTickPosition(QSlider::TicksBelow);
        kcfg_SnappingPrecision->setTickInterval(0);

        gridLayout_3->addWidget(kcfg_SnappingPrecision, 0, 1, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(groupBox_3);
        label->setObjectName("label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(label);

        horizontalSpacer = new QSpacerItem(40, 0, QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        label_2 = new QLabel(groupBox_3);
        label_2->setObjectName("label_2");
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(label_2);


        gridLayout_3->addLayout(horizontalLayout, 1, 1, 1, 1);


        verticalLayout_2->addWidget(groupBox_3);

        verticalSpacer = new QSpacerItem(20, 34, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        retranslateUi(Settings);
        QObject::connect(kcfg_ViewBackground, SIGNAL(itemTypeChanged(bool)), kcfg_ViewBackgroundColor, SLOT(setEnabled(bool)));

        QMetaObject::connectSlotsByName(Settings);
    } // setupUi

    void retranslateUi(QWidget *Settings)
    {
        groupBox_2->setTitle(tr2i18n("Puzzle table", "@title:group"));
        label_3->setText(tr2i18n("Background:", "@label:chooser for different types of background textures"));
        label_7->setText(tr2i18n("Color for highlighting selected pieces:", nullptr));
        label_5->setText(tr2i18n("Space for solution:", nullptr));
        label_6->setText(tr2i18n("No space is provided with puzzles of less than 20 pieces. Changes will take effect when a puzzle is created or re-started.", nullptr));
        label_8->setText(tr2i18n("Spacing of pieces in puzzle grids (1.0-1.5):", nullptr));
        groupBox->setTitle(tr2i18n("Piece appearance", nullptr));
        kcfg_PieceBevelsEnabled->setText(tr2i18n("Draw bevels to create a 3-dimensional appearance", nullptr));
        kcfg_PieceShadowsEnabled->setText(tr2i18n("Draw shadows below pieces", nullptr));
        label_4->setText(tr2i18n("These options can be disabled when Palapeli is running very slowly. Changes will take effect only when a new puzzle is started.", nullptr));
        groupBox_3->setTitle(tr2i18n("Piece behavior", "@title:group"));
        label_1->setText(tr2i18n("Snapping precision:", "@label:slider"));
        label->setText(tr2i18n("Very precise (harder)", "@item:inrange description for the \"snapping precision\" setting"));
        label_2->setText(tr2i18n("Unprecise (easier)", "@item:inrange description for the \"snapping precision\" setting"));
        (void)Settings;
    } // retranslateUi

};

namespace Ui {
    class Settings: public Ui_Settings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SETTINGS_H

