/****************************************************************************
** Meta object code from reading C++ file 'gameplay.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../palapeli-26.07.80/src/engine/gameplay.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gameplay.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8Palapeli8GamePlayE_t {};
} // unnamed namespace

template <> constexpr inline auto Palapeli::GamePlay::qt_create_metaobjectdata<qt_meta_tag_ZN8Palapeli8GamePlayE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Palapeli::GamePlay",
        "reportProgress",
        "",
        "pieceCount",
        "originalCount",
        "victoryAnimationFinished",
        "playPuzzle",
        "Palapeli::Puzzle*",
        "puzzle",
        "playPuzzleFile",
        "path",
        "actionGoCollection",
        "actionTogglePreview",
        "actionCreate",
        "actionDelete",
        "actionImport",
        "actionExport",
        "createHolder",
        "deleteHolder",
        "selectAll",
        "rearrangePieces",
        "actionZoomIn",
        "actionZoomOut",
        "restartPuzzle",
        "configure",
        "positionChanged",
        "reduction",
        "loadPreview",
        "loadPuzzleFile",
        "loadNextPiece",
        "loadPiecePositions",
        "finishLoading",
        "playVictoryAnimation2",
        "playVictoryAnimation3",
        "updateSavedGame",
        "changeSelectedHolder",
        "PieceHolder*",
        "h",
        "teleport",
        "Piece*",
        "piece",
        "QPointF",
        "scenePos",
        "View*",
        "view",
        "closeHolder",
        "handleNewPieceSelection"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'reportProgress'
        QtMocHelpers::SignalData<void(int, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'victoryAnimationFinished'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'playPuzzle'
        QtMocHelpers::SlotData<void(Palapeli::Puzzle *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'playPuzzleFile'
        QtMocHelpers::SlotData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'actionGoCollection'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'actionTogglePreview'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'actionCreate'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'actionDelete'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'actionImport'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'actionExport'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'createHolder'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteHolder'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectAll'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'rearrangePieces'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'actionZoomIn'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'actionZoomOut'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'restartPuzzle'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'configure'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'positionChanged'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 },
        }}),
        // Slot 'loadPreview'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'loadPuzzleFile'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'loadNextPiece'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'loadPiecePositions'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'finishLoading'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'playVictoryAnimation2'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'playVictoryAnimation3'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateSavedGame'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'changeSelectedHolder'
        QtMocHelpers::SlotData<void(PieceHolder *)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'teleport'
        QtMocHelpers::SlotData<void(Piece *, const QPointF &, View *)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 39, 40 }, { 0x80000000 | 41, 42 }, { 0x80000000 | 43, 44 },
        }}),
        // Slot 'closeHolder'
        QtMocHelpers::SlotData<void(PieceHolder *)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'handleNewPieceSelection'
        QtMocHelpers::SlotData<void(View *)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 44 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GamePlay, qt_meta_tag_ZN8Palapeli8GamePlayE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Palapeli::GamePlay::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Palapeli8GamePlayE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Palapeli8GamePlayE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8Palapeli8GamePlayE_t>.metaTypes,
    nullptr
} };

void Palapeli::GamePlay::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GamePlay *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->reportProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->victoryAnimationFinished(); break;
        case 2: _t->playPuzzle((*reinterpret_cast<std::add_pointer_t<Palapeli::Puzzle*>>(_a[1]))); break;
        case 3: _t->playPuzzleFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->actionGoCollection(); break;
        case 5: _t->actionTogglePreview(); break;
        case 6: _t->actionCreate(); break;
        case 7: _t->actionDelete(); break;
        case 8: _t->actionImport(); break;
        case 9: _t->actionExport(); break;
        case 10: _t->createHolder(); break;
        case 11: _t->deleteHolder(); break;
        case 12: _t->selectAll(); break;
        case 13: _t->rearrangePieces(); break;
        case 14: _t->actionZoomIn(); break;
        case 15: _t->actionZoomOut(); break;
        case 16: _t->restartPuzzle(); break;
        case 17: _t->configure(); break;
        case 18: _t->positionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->loadPreview(); break;
        case 20: _t->loadPuzzleFile(); break;
        case 21: _t->loadNextPiece(); break;
        case 22: _t->loadPiecePositions(); break;
        case 23: _t->finishLoading(); break;
        case 24: _t->playVictoryAnimation2(); break;
        case 25: _t->playVictoryAnimation3(); break;
        case 26: _t->updateSavedGame(); break;
        case 27: _t->changeSelectedHolder((*reinterpret_cast<std::add_pointer_t<PieceHolder*>>(_a[1]))); break;
        case 28: _t->teleport((*reinterpret_cast<std::add_pointer_t<Piece*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<View*>>(_a[3]))); break;
        case 29: _t->closeHolder((*reinterpret_cast<std::add_pointer_t<PieceHolder*>>(_a[1]))); break;
        case 30: _t->handleNewPieceSelection((*reinterpret_cast<std::add_pointer_t<View*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GamePlay::*)(int , int )>(_a, &GamePlay::reportProgress, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GamePlay::*)()>(_a, &GamePlay::victoryAnimationFinished, 1))
            return;
    }
}

const QMetaObject *Palapeli::GamePlay::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Palapeli::GamePlay::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Palapeli8GamePlayE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Palapeli::GamePlay::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 31;
    }
    return _id;
}

// SIGNAL 0
void Palapeli::GamePlay::reportProgress(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void Palapeli::GamePlay::victoryAnimationFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
